function dosyplot_gui(dosydata,clev,thresh)

%   dosyplot_gui(dosydata,clev,thresh);
%   dosyplot_gui.m: plots dosy data as obtained from dosy_mn.m, in an
%   interactive graphical user interface
%
%   --------------------------------INPUT------------------------------------
%   dosydata  =  The data structure from dosy_mn
%   clev      =   Number of contour levels
%   thresh    =   Threshold for contour levels as per cent of max peak height
%                 a negative value will set the default value (0.1 1% of
%                 max peak aplitude)
%
%
%   Example:
%
%   See also: dosy_mn, score_mn, decra_mn, mcr_mn, varianimport,
%             brukerimport, jeolimport, peakpick_mn, dosyplot_mn,
%             dosyresidual, dosyplot_gui, scoreplot_mn, decraplot_mn,
%             mcrplot_mn
%
%   This is a part of the DOSYToolbox
%   Copyright  2007-2008  <Mathias Nilsson>%
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
%
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
%
%   You should have received a copy of the GNU General Public License along
%   with this program; if not, write to the Free Software Foundation, Inc.,
%   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
%
%   Dr. Mathias Nilsson
%   School of Chemistry, University of Manchester,
%   Oxford Road, Manchester M13 9PL, UK
%   Telephone: +44 (0) 161 306 4465
%   Fax: +44 (0)161 275 4598

if nargin==0
    disp(' ')
    disp(' ')
    disp(' DOSYPLOT')
    disp('At least one input is needed')
    disp(' ')
    disp(' Type <<help dosyplot>> for more info')
    disp('  ')
    return
end
if nargin >6
    error(' Too many inputs')
end

if nargin>1
    if clev<=0
        clev=7;
    end
else
    clev=7;
end
clvl=2.^(0:clev);
clvl=clvl(1:clev);
clvl=1./clvl;
clvl=[clvl -clvl];
if nargin > 2
    if thresh<0
        thresh=0.1;
    end
else
    if (dosydata.threshold)
        thresh=dosydata.threshold;
    else
        thresh=0.1;
    end
end


%-----GUI CONTROLS----------------------------------------------------------
%--------------------------------------------------------------------------


hMainFigure = figure(...
    'Units','pixels',...
    'MenuBar','figure',...
    'Name','DOSY Plotting',...
    'NumberTitle','Off',...
    'Toolbar','Figure',...
    'OuterPosition',[0.0 0.0 1280 1024 ],...
     'Visible','off');
set(hMainFigure,'Units','Normalized');
set(hMainFigure, 'Position',[0.1 0.4 0.6 0.6]);
hDOSY = axes('Units','normalized',...
    'Position',[0.1 0.34 0.7 0.5]);
hSpec = axes('Units','normalized',...
    'Position',[0.1 0.85 0.7 0.1]);
hProj = axes('Units','normalized',...
    'Position',[0.82 0.34 0.1 0.5]);


%-----Panel for Projections-----------------------------------------------
hProjectPanel=uipanel(...
    'Parent',hMainFigure,...
    'Title','Projections',...
    'FontWeight','bold',...
    'ForegroundColor','Blue',...
    'TitlePosition','centertop',...
    'Units','Normalized',...
    'Position',[0.425,0.01,0.1,0.25]);

hDrawBoxes = uicontrol(...
    'Parent',hProjectPanel,...
    'Style','PushButton',...
    'String','Draw',...
    'Units','normalized',...
    'Position',[0.1 0.7 0.8 0.2],...
    'Callback',@DrawBoxButton_Callback);

hClearBoxes = uicontrol(...
    'Parent',hProjectPanel,...
    'Style','PushButton',...
    'String','Clear',...
    'Units','normalized',...
    'Position',[0.1 0.4 0.8 0.2],...
    'Callback',@ClearBoxButton_Callback);

hPlotBoxes = uicontrol(...
    'Parent',hProjectPanel,...
    'Style','PushButton',...
    'String','Plot',...
    'Units','normalized',...
    'Position',[0.1 0.1 0.8 0.2],...
    'Callback',@PlotBoxesButton_Callback);



%-----Panel for Plot Control-----------------------------------------------
hPlotPanel=uipanel(...
    'Parent',hMainFigure,...
    'Title','Plot Control',...
    'FontWeight','bold',...
    'ForegroundColor','Blue',...
    'TitlePosition','centertop',...
    'Units','Normalized',...
    'Position',[0.01,0.01,0.4,0.25]);
% hlogscale = uicontrol(...
%     'Parent',hPlotPanel,...
%     'Style','PushButton',...
%     'String','LogScale',...
%     'Units','normalized',...
%     'Position',[0.05 0.65 0.1 0.1],...
%     'Callback',@logscaleButton_Callback);%#ok
hlogscaleGroup=uibuttongroup(...
    'Visible','On',...
    'Parent',hPlotPanel,...
    'Title','D scale',...
    'SelectionChangeFcn', {@Scale_Callback},...
    'TitlePosition','centertop',...
    'Units','Normalized',...
    'Position',[0.4 0.5 0.2 0.3]);
hRadioLog = uicontrol(...
    'Parent',hlogscaleGroup,...
    'Style','RadioButton',...
    'String','log',...
    'Units','normalized',...
    'Position',[0.05 0.3 0.4 0.7]);
hRadioLin = uicontrol(...
    'Parent',hlogscaleGroup,...
    'Style','RadioButton',...
    'String','lin',...
    'Value',1,...
    'Units','normalized',...
    'Position',[0.55 0.3 0.4 0.7]);
hTextZoom=uicontrol(...
    'Parent',hPlotPanel,...
    'Style','text',...
    'Units','Normalized',...
    'String','Zoom',...
    'Position',[0.15 0.9 0.2 0.1 ],...
    'horizontalalignment','c');%#ok
hZoomXYButton = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String','XY',...
    'Units','normalized',...
    'Position',[0.05 0.8 0.1 0.1],...
    'Callback',@ZoomXYButton_Callback);%#ok
hZoomXButton = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String','X',...
    'Units','normalized',...
    'Position',[0.15 0.8 0.1 0.1],...
    'Callback',@ZoomXButton_Callback);%#ok
hZoomYButton = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String','Y',...
    'Units','normalized',...
    'Position',[0.25 0.8 0.1 0.1],...
    'Callback',@ZoomYButton_Callback);%#ok
hZoomOffButton = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String','off',...
    'Units','normalized',...
    'Position',[0.35 0.8 0.1 0.1],...
    'Callback',@ZoomOffButton_Callback);%#ok
hTextpan=uicontrol(...
    'Parent',hPlotPanel,...
    'Style','text',...
    'Units','Normalized',...
    'String','Pan',...
    'Position',[0.65 0.9 0.2 0.1 ],...
    'horizontalalignment','c');%#ok
hpanXYButton = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String','XY',...
    'Units','normalized',...
    'Position',[0.55 0.8 0.1 0.1],...
    'Callback',@panXYButton_Callback);%#ok
hpanXButton = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String','X',...
    'Units','normalized',...
    'Position',[0.65 0.8 0.1 0.1],...
    'Callback',@panXButton_Callback);%#ok
hpanYButton = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String','Y',...
    'Units','normalized',...
    'Position',[0.75 0.8 0.1 0.1],...
    'Callback',@panYButton_Callback);%#ok
hpanOffButton = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String','off',...
    'Units','normalized',...
    'Position',[0.85 0.8 0.1 0.1],...
    'Callback',@panOffButton_Callback);%#ok

hTextScale=uicontrol(...
    'Parent',hPlotPanel,...
    'Style','text',...
    'Units','Normalized',...
    'String','Scale',...
    'Position',[0.1 0.55 0.2 0.1 ],...
    'horizontalalignment','c');%#ok
hButtonMult2 = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String',[char(hex2dec('D7')) ' 2'],...
    'Units','Normalized',...
    'Position',[0.1 0.3 0.1 0.15],...
    'Callback',{@ButtonMult2_Callback});%#ok
hButtonDiv2 = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String',[char(hex2dec('F7')) ' 2'],...
    'Units','Normalized',...
    'Position',[0.1 0.10 0.1 0.15],...
    'Callback',{@ButtonDiv2_Callback});%#ok
hButtonMult11 = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String',[char(hex2dec('D7')) ' 1.1'],...
    'Units','Normalized',...
    'Position',[0.2 0.3 0.1 0.15],...
    'Callback',{@ButtonMult11_Callback});%#ok
hButtonDiv11 = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','PushButton',...
    'String',[char(hex2dec('F7')) ' 1.1'],...
    'Units','Normalized',...
    'Position',[0.2 0.10 0.1 0.15],...
    'Callback',{@ButtonDiv11_Callback});%#ok
hButtonOrgscale = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','Pushbutton',...
    'String','Original',...
    'Units','Normalized',...
    'Position',[0.4 0.3 0.2 0.15],...
    'Callback',{@ButtonOrgscale_Callback});%#ok
hButtonReplot = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','Pushbutton',...
    'String','Replot',...
    'Units','Normalized',...
    'Position',[0.4 0.1 0.2 0.15],...
    'Callback',{@ButtonReplot_Callback});%#ok

hButtonPlotSep = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','Pushbutton',...
    'String','Separate Plot',...
    'Units','Normalized',...
    'Position',[0.7 0.3 0.2 0.15],...
    'Callback',{@ButtonPlotSep_Callback});%#ok
hButtonPlotPub = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','Pushbutton',...
    'String','Publication Plot',...
    'Units','Normalized',...
    'Position',[0.7 0.5 0.2 0.15],...
    'Callback',{@ButtonPlotPub_Callback});%#ok

hButtonStats = uicontrol(...
    'Parent',hPlotPanel,...
    'Style','Pushbutton',...
    'String','Stats',...
    'Units','Normalized',...
    'Position',[0.7 0.1 0.2 0.15],...
    'Callback',{@ButtonStats_Callback});%#ok


%-----Panel for DOSY residuals-----------------------------------------------
hResidPanel=uipanel(...
    'Parent',hMainFigure,...
    'Title','Residuals',...
    'FontWeight','bold',...
    'ForegroundColor','Blue',...
    'TitlePosition','centertop',...
    'Visible','Off',...
    'Units','Normalized',...
    'Position',[0.55,0.01,0.4,0.25]);
hButtonPlotResid = uicontrol(...
    'Parent',hResidPanel,...
    'Style','PushButton',...
    'String','Plot Residual',...
    'Units','normalized',...
    'Position',[0.1 0.8 0.28 0.15],...
    'Callback',{@ButtonPlotResid_Callback});%#ok
hButtonPlotResidSep = uicontrol(...
    'Parent',hResidPanel,...
    'Style','PushButton',...
    'String','Separate Plot',...
    'Units','normalized',...
    'Position',[0.1 0.5 0.28 0.15],...
    'Callback',{@ButtonPlotResidSep_Callback});%#ok
hEditFlip=uicontrol(...
    'Parent',hResidPanel,...
    'Style','edit',...
    'Units','Normalized',...
    'BackgroundColor','w',...
    'String',1,...
    'Position',[0.18 0.1 0.1 0.1 ],...
    'CallBack', {@EditFlip_Callback});
hButtonFlipPlus = uicontrol(...
    'Parent',hResidPanel,...
    'Style','PushButton',...
    'String','+1',...
    'Units','normalized',...
    'Position',[0.3 0.1 0.06 0.1],...
    'Callback',{@ButtonFlipPlus_Callback});                            %#ok
hButtonFlipMinus = uicontrol(...
    'Parent',hResidPanel,...
    'Style','PushButton',...
    'String','-1',...
    'Units','normalized',...
    'Position',[0.1 0.1 0.06 0.1],...
    'Callback',{@ButtonFlipMinus_Callback});                           %#ok
hTextPeakNumber=uicontrol(...
    'Parent',hResidPanel,...
    'Style','text',...
    'Units','Normalized',...
    'String','Peak Number',...
    'FontWeight','bold',...
    'Position',[0.03 0.25 0.4 0.1 ],...
    'horizontalalignment','c');%#ok

hButtonCompareResid = uicontrol(...
    'Parent',hResidPanel,...
    'Style','PushButton',...
    'String','Compare Residuals',...
    'Units','normalized',...
    'Position',[0.6 0.8 0.28 0.15],...
    'Callback',{@ButtonCompareResid_Callback});%#ok
hEditComp1=uicontrol(...
    'Parent',hResidPanel,...
    'Style','edit',...
    'Units','Normalized',...
    'BackgroundColor','w',...
    'Enable','Off',...
    'String',1,...
    'Position',[0.6 0.62 0.1 0.1 ],...
    'CallBack', {@EditComp1_Callback});
hEditComp2=uicontrol(...
    'Parent',hResidPanel,...
    'Style','edit',...
    'Units','Normalized',...
    'BackgroundColor','w',...
    'Enable','Off',...
    'String',1,...
    'Position',[0.6 0.5 0.1 0.1 ],...
    'CallBack', {@EditComp2_Callback});
hEditComp3=uicontrol(...
    'Parent',hResidPanel,...
    'Style','edit',...
    'Units','Normalized',...
    'BackgroundColor','w',...
    'Enable','Off',...
    'String',1,...
    'Position',[0.6 0.38 0.1 0.1 ],...
    'CallBack', {@EditComp3_Callback});
hEditComp4=uicontrol(...
    'Parent',hResidPanel,...
    'Style','edit',...
    'Units','Normalized',...
    'BackgroundColor','w',...
    'Enable','Off',...
    'String',1,...
    'Position',[0.6 0.26 0.1 0.1 ],...
    'CallBack', {@EditComp4_Callback});
hEditComp5=uicontrol(...
    'Parent',hResidPanel,...
    'Style','edit',...
    'Units','Normalized',...
    'BackgroundColor','w',...
    'Enable','Off',...
    'String',1,...
    'Position',[0.6 0.14 0.1 0.1 ],...
    'CallBack', {@EditComp5_Callback});
hCheckComp1 = uicontrol(...
    'Parent',hResidPanel,...
    'Style','Checkbox',...
    'String','',...
    'Units','normalized',...
    'Value',0,...
    'Position',[0.54 0.62 0.05 0.1],...
    'Callback',{@CheckComp1_Callback});
hCheckComp2 = uicontrol(...
    'Parent',hResidPanel,...
    'Style','Checkbox',...
    'String','',...
    'Units','normalized',...
    'Value',0,...
    'Position',[0.54 0.50 0.05 0.1],...
    'Callback',{@CheckComp2_Callback});
hCheckComp3 = uicontrol(...
    'Parent',hResidPanel,...
    'Style','Checkbox',...
    'String','',...
    'Units','normalized',...
    'Value',0,...
    'Position',[0.54 0.38 0.05 0.1],...
    'Callback',{@CheckComp3_Callback});
hCheckComp4 = uicontrol(...
    'Parent',hResidPanel,...
    'Style','Checkbox',...
    'String','',...
    'Units','normalized',...
    'Value',0,...
    'Position',[0.54 0.26 0.05 0.1],...
    'Callback',{@CheckComp4_Callback});
hCheckComp5 = uicontrol(...
    'Parent',hResidPanel,...
    'Style','Checkbox',...
    'String','',...
    'Units','normalized',...
    'Value',0,...
    'Position',[0.54 0.14 0.05 0.1],...
    'Callback',{@CheckComp5_Callback});
hTextData=uicontrol(...
    'Parent',hResidPanel,...
    'Style','text',...
    'Units','Normalized',...
    'String','Data Type',...
    'FontWeight','bold',...
    'Position',[0.7 0.7 0.3 0.1 ],...
    'horizontalalignment','c');%#ok
hCheckOrig = uicontrol(...
    'Parent',hResidPanel,...
    'Style','Checkbox',...
    'String','Original',...
    'Units','normalized',...
    'Value',0,...
    'Position',[0.75 0.50 0.2 0.1],...
    'Callback',{@CheckOrig_Callback});
hCheckFitted = uicontrol(...
    'Parent',hResidPanel,...
    'Style','Checkbox',...
    'String','Fitted',...
    'Units','normalized',...
    'Value',0,...
    'Position',[0.75 0.35 0.2 0.1],...
    'Callback',{@CheckFitted_Callback});
hCheckResid = uicontrol(...
    'Parent',hResidPanel,...
    'Style','Checkbox',...
    'String','Residual',...
    'Units','normalized',...
    'Value',0,...
    'Position',[0.75 0.20 0.2 0.1],...
    'Callback',{@CheckResid_Callback});




if strcmp(dosydata.type,'dosy') || strcmp(dosydata.type,'ILT')
    set(hResidPanel,'Visible','On');
end
%-----End Panel for the flipping through spectra---------------------------
maxc=max(max(dosydata.DOSY));
dosyguidata.clvl=clvl.*maxc.*(thresh/100);
dosyguidata.residpeak=1;
dosyguidata.comparepeaks=[1 1 1 1 1];
dosyguidata.compareuse=[0 0 0 0 0];
dosyguidata.comparetype=[0 0 0];

%dosydatagui.clvl=clvl/2;
guidata(hMainFigure,dosyguidata);


PlotSpectrum();
PlotContour();
PlotProjection();


movegui(hMainFigure,'center')
movegui(hMainFigure,'north')
set(hMainFigure,'Visible','on')



%---------Plot Control Callbacks-------------------------------------------
    function ZoomXYButton_Callback(source,eventdata)%#ok
        zoom off
        pan off
        zoom on       
    end
    function ZoomXButton_Callback(source,eventdata)%#ok
        zoom off
        pan off
        zoom xon
    end
    function ZoomYButton_Callback(source,eventdata)%#ok
        zoom off
        pan off
        zoom yon
    end
    function ZoomOffButton_Callback(source,eventdata)%#ok
        zoom off
        pan off
    end
    function panXYButton_Callback(source,eventdata)%#ok
        pan off
        zoom off
        pan on
    end
    function panXButton_Callback(source,eventdata)%#ok
        pan off
        zoom off
        pan xon
    end
    function panYButton_Callback(source,eventdata)%#ok
        pan off
        zoom off
        pan yon
    end
    function panOffButton_Callback(source,eventdata)%#ok
        pan off
        zoom off
    end
    function ButtonMult2_Callback(source,eventdata)%#ok       
        dosyguidata=guidata(hMainFigure);
        dosyguidata.clvl=dosyguidata.clvl/2;
        guidata(hMainFigure,dosyguidata);   
        newz=get(dosyguidata.hContour1,'Zdata') * 2;        
        set(dosyguidata.hContour1,'Zdata',newz);        
        newz=get(dosyguidata.hContour2,'Zdata') * 2;        
        set(dosyguidata.hContour2,'Zdata',newz);          
    end
    function ButtonMult11_Callback(source,eventdata)%#ok
        dosyguidata=guidata(hMainFigure);
        dosyguidata.clvl=dosyguidata.clvl/1.1;
        guidata(hMainFigure,dosyguidata);
        newz=get(dosyguidata.hContour1,'Zdata') * 1.1;        
        set(dosyguidata.hContour1,'Zdata',newz);        
        newz=get(dosyguidata.hContour2,'Zdata') * 1.1;        
        set(dosyguidata.hContour2,'Zdata',newz);       
    end
    function ButtonDiv2_Callback(source,eventdata)%#ok       
        dosyguidata=guidata(hMainFigure);
        dosyguidata.clvl=dosyguidata.clvl*2;
        guidata(hMainFigure,dosyguidata);
        newz=get(dosyguidata.hContour1,'Zdata') / 2;        
        set(dosyguidata.hContour1,'Zdata',newz);        
        newz=get(dosyguidata.hContour2,'Zdata') /2;        
        set(dosyguidata.hContour2,'Zdata',newz);   
    end
    function ButtonDiv11_Callback(source,eventdata)%#ok
        dosyguidata=guidata(hMainFigure);
        dosyguidata.clvl=dosyguidata.clvl*1.1;
        guidata(hMainFigure,dosyguidata);
        newz=get(dosyguidata.hContour1,'Zdata') / 1.1;        
        set(dosyguidata.hContour1,'Zdata',newz);        
        newz=get(dosyguidata.hContour2,'Zdata') /1.1;        
        set(dosyguidata.hContour2,'Zdata',newz);
    end
    function ButtonOrgscale_Callback(source,eventdata) %#ok
        dosyguidata=guidata(hMainFigure);        
        maxc=max(max(dosydata.DOSY));
        dosyguidata.clvl=clvl.*maxc.*(thresh/100);
        guidata(hMainFigure,dosyguidata);
        [xax,yax]=meshgrid(dosydata.Ppmscale,dosydata.Dscale);
        [C,hContour1]=contour(hDOSY,xax,yax,(dosydata.DOSY.*(dosydata.DOSY>0))',dosyguidata.clvl,'b');
        hold(hDOSY,'on')
        [C,hContour2]=contour(hDOSY,xax,yax,(dosydata.DOSY.*(dosydata.DOSY<0))',dosyguidata.clvl,'color',[1 0.5 0]);
        hold(hDOSY,'off')
        set(hContour1,'LineWidth',1)
        set(hContour2,'LineWidth',1)
        set(hDOSY,'LineWidth',1)
        set(hDOSY,'Ydir','reverse');
        ylabel(hDOSY,'\bf Diffusion coefficient m^{2} s^{-1} \times 10^{-10}');
        set(hDOSY,'Xdir','reverse');
        xlabel(hDOSY,'\bf Chemical shift (PPM)');   
         dosyguidata.hContour1=hContour1;
        dosyguidata.hContour2=hContour2;
        guidata(hMainFigure,dosyguidata);
    end
    function ButtonReplot_Callback(source,eventdata) %#ok
        dosyguidata=guidata(hMainFigure);
        PlotContour();
            end
    function ButtonPlotSep_Callback(source,eventdata) %#ok
        dosyguidata=guidata(hMainFigure);
        % Create plotdata to fit the zooming.
        dosydata.xlim=xlim(hDOSY);
        dosydata.ylim=ylim(hDOSY);
        dosyplot(dosydata,dosyguidata.clvl);
    end
    function ButtonPlotPub_Callback(source,eventdata) %#ok
        dosyguidata=guidata(hMainFigure);
        % Create plotdata to fit the zooming.
        dosydata.xlim=xlim(hDOSY);
        dosydata.ylim=ylim(hDOSY);
        dosyplotpub(dosydata,dosyguidata.clvl);
    end
    function ButtonStats_Callback(source,eventdata)
        DTpath=which('DOSYToolbox');
        DTpath=DTpath(1:(end-13));
        %open([DTpath 'dosystats.txt']);
        type([DTpath 'dosystats.txt']);
        
    end
	function logscaleButton_Callback(source,eventdata)
		dosyguidata=guidata(hMainFigure);
        	set(hDOSY,'YScale','log'); 
    end
    function Scale_Callback(source,eventdata)
        switch get(hlogscaleGroup,'SelectedObject')
            case hRadioLog
                dosyguidata=guidata(hMainFigure);
                set(hDOSY,'YScale','log');
            case hRadioLin
                dosyguidata=guidata(hMainFigure);
                set(hDOSY,'YScale','linear');
            otherwise
                error('illegal choice')
        end
        
    end
%---------End of Plot Control Callbacks------------------------------------

%---------Plot Residual Callbacks------------------------------------------
    function ButtonPlotResid_Callback(source,eventdata) %#ok
        dosyguidata=guidata(hMainFigure);
        PlotResidual(dosyguidata.residpeak);
    end
    function ButtonPlotResidSep_Callback(source,eventdata) %#ok
        dosyguidata=guidata(hMainFigure);
        dosyresidual(dosydata,dosyguidata);
    end
    function EditComp1_Callback(source,eventdata) %#ok<INUSD>
        dosyguidata=guidata(hMainFigure);
        
        if isnan((str2double(get(hEditComp1,'string'))))
            disp('Must be a number')
            set(hEditComp1,'String',num2str(dosyguidata.comparepeaks(1)))
        else
            
            dosyguidata.comparepeaks(1)=...
                round((str2double(get(hEditComp1,'string'))));
            
            if dosyguidata.comparepeaks(1)>length(dosydata.freqs)
                dosyguidata.comparepeaks(1)=length(dosydata.freqs);
                set(hEditComp1,'String',num2str(dosyguidata.comparepeaks(1)))
            end
            if dosyguidata.comparepeaks(1)<0
                dosyguidata.comparepeaks(1)=1;
                set(hEditComp1,'String',num2str(dosyguidata.comparepeaks(1)))
            end
        end
        guidata(hMainFigure,dosyguidata);
    end
    function EditComp2_Callback(source,eventdata) %#ok<INUSD>
        dosyguidata=guidata(hMainFigure);
        if isnan((str2double(get(hEditComp2,'string'))))
            disp('Must be a number')
            set(hEditComp2,'String',num2str(dosyguidata.comparepeaks(2)))
        else
            dosyguidata.comparepeaks(2)=...
                round((str2double(get(hEditComp1,'string'))));
            
            if dosyguidata.comparepeaks(2)>length(dosydata.freqs)
                dosyguidata.comparepeaks(2)=length(dosydata.freqs);
                set(hEditComp1,'String',num2str(dosyguidata.comparepeaks(2)))
            end
            if dosyguidata.comparepeaks(2)<0
                dosyguidata.comparepeaks(2)=1;
                set(hEditComp1,'String',num2str(dosyguidata.comparepeaks(2)))
            end
        end
        guidata(hMainFigure,dosyguidata);
    end
    function EditComp3_Callback(source,eventdata) %#ok<INUSD>
        dosyguidata=guidata(hMainFigure);
        if isnan((str2double(get(hEditComp3,'string'))))
            disp('Must be a number')
            set(hEditComp3,'String',num2str(dosyguidata.comparepeaks(3)))
        else
            dosyguidata.comparepeaks(3)=...
                round((str2double(get(hEditComp3,'string'))));
            
            if dosyguidata.comparepeaks(3)>length(dosydata.freqs)
                dosyguidata.comparepeaks(3)=length(dosydata.freqs);
                set(hEditComp3,'String',num2str(dosyguidata.comparepeaks(3)))
            end
            if dosyguidata.comparepeaks(3)<0
                dosyguidata.comparepeaks(3)=1;
                set(hEditComp3,'String',num2str(dosyguidata.comparepeaks(3)))
            end
        end
        guidata(hMainFigure,dosyguidata);
    end
    function EditComp4_Callback(source,eventdata) %#ok<INUSD>
        dosyguidata=guidata(hMainFigure);
        if isnan((str2double(get(hEditComp4,'string'))))
            disp('Must be a number')
            set(hEditComp4,'String',num2str(dosyguidata.comparepeaks(4)))
        else
            dosyguidata.comparepeaks(4)=...
                round((str2double(get(hEditComp4,'string'))));
            
            if dosyguidata.comparepeaks(4)>length(dosydata.freqs)
                dosyguidata.comparepeaks(4)=length(dosydata.freqs);
                set(hEditComp4,'String',num2str(dosyguidata.comparepeaks(4)))
            end
            if dosyguidata.comparepeaks(4)<0
                dosyguidata.comparepeaks(4)=1;
                set(hEditComp4,'String',num2str(dosyguidata.comparepeaks(4)))
            end
        end
        guidata(hMainFigure,dosyguidata);
    end
    function EditComp5_Callback(source,eventdata) %#ok<INUSD>
        dosyguidata=guidata(hMainFigure);
        if isnan((str2double(get(hEditComp5,'string'))))
            disp('Must be a number')
            set(hEditComp5,'String',num2str(dosyguidata.comparepeaks(5)))
        else
            dosyguidata.comparepeaks(5)=...
                round((str2double(get(hEditComp5,'string'))));
            
            if dosyguidata.comparepeaks(5)>length(dosydata.freqs)
                dosyguidata.comparepeaks(5)=length(dosydata.freqs);
                set(hEditComp5,'String',num2str(dosyguidata.comparepeaks(5)))
            end
            if dosyguidata.comparepeaks(5)<0
                dosyguidata.comparepeaks(5)=1;
                set(hEditComp5,'String',num2str(dosyguidata.comparepeaks(5)))
            end
        end
        guidata(hMainFigure,dosyguidata);
    end
    function CheckComp1_Callback(source,eventdata)  %#ok<INUSD>
        if get(hCheckComp1,'Value')                   %#ok
            set(hEditComp1,'Enable','On')
            dosyguidata.compareuse(1)=1;
        else
            set(hEditComp1,'Enable','Off')
            dosyguidata.compareuse(1)=0;
        end
        guidata(hMainFigure,dosyguidata);
    end
    function CheckComp2_Callback(source,eventdata)  %#ok<INUSD>
        if get(hCheckComp2,'Value')                   %#ok
            set(hEditComp2,'Enable','On')
            dosyguidata.compareuse(2)=1;
        else
            set(hEditComp2,'Enable','Off')
            dosyguidata.compareuse(2)=0;
        end
        guidata(hMainFigure,dosyguidata);
    end
    function CheckComp3_Callback(source,eventdata)  %#ok<INUSD>
        if get(hCheckComp3,'Value')                   %#ok
            set(hEditComp3,'Enable','On')
            dosyguidata.compareuse(3)=1;
        else
            set(hEditComp3,'Enable','Off')
            dosyguidata.compareuse(3)=0;
        end
        guidata(hMainFigure,dosyguidata);
    end
    function CheckComp4_Callback(source,eventdata)  %#ok<INUSD>
        if get(hCheckComp4,'Value')                   %#ok
            set(hEditComp4,'Enable','On')
            dosyguidata.compareuse(4)=1;
        else
            set(hEditComp4,'Enable','Off')
            dosyguidata.compareuse(4)=0;
        end
        guidata(hMainFigure,dosyguidata);
    end
    function CheckComp5_Callback(source,eventdata)  %#ok<INUSD>
        if get(hCheckComp5,'Value')                   %#ok
            set(hEditComp5,'Enable','On')
            dosyguidata.compareuse(5)=1;
        else
            set(hEditComp5,'Enable','Off')
            dosyguidata.compareuse(5)=0;
        end
        guidata(hMainFigure,dosyguidata);
    end
    function CheckOrig_Callback(source,eventdata)  %#ok<INUSD>
        if get(hCheckOrig,'Value')                   %#ok
            dosyguidata.comparetype(1)=1;
        else
            dosyguidata.comparetype(1)=0;
        end
        guidata(hMainFigure,dosyguidata);
    end
    function CheckFitted_Callback(source,eventdata)  %#ok<INUSD>
        if get(hCheckFitted,'Value')                   %#ok
            dosyguidata.comparetype(2)=1;
        else
            dosyguidata.comparetype(2)=0;
        end
        guidata(hMainFigure,dosyguidata);
    end
    function CheckResid_Callback(source,eventdata)  %#ok<INUSD>
        if get(hCheckFitted,'Value')                   %#ok
            dosyguidata.comparetype(3)=1;
        else
            dosyguidata.comparetype(3)=0;
        end
        guidata(hMainFigure,dosyguidata);
    end
    function ButtonCompareResid_Callback(source,eventdata) %#ok<INUSD>
        dosyguidata=guidata(hMainFigure);
        dosyresidualcompare(dosydata,dosyguidata);
    end

    function EditFlip_Callback(source,eventdata)                       %#ok
        
        dosyguidata=guidata(hMainFigure);
        dosyguidata.residpeak=round(str2double(get(hEditFlip,'String')));
        if dosyguidata.residpeak>length(dosydata.freqs)
            dosyguidata.residpeak=length(dosydata.freqs);
            set(hEditFlip,'String',num2str(dosyguidata.residpeak))
        end
        if dosyguidata.residpeak<1
            dosyguidata.residpeak=1;
            set(hEditFlip,'String',num2str(dosyguidata.residpeak))
        end
        guidata(hMainFigure,dosyguidata);
        PlotResidual(dosyguidata.residpeak);
    end
    function ButtonFlipPlus_Callback(source,eventdata) %#ok
        dosyguidata=guidata(hMainFigure);
        dosyguidata.residpeak=dosyguidata.residpeak+1;
        if dosyguidata.residpeak>length(dosydata.freqs)
            dosyguidata.residpeak=length(dosydata.freqs);
            set(hEditFlip,'String',num2str(dosyguidata.residpeak))
        end
        if dosyguidata.residpeak<1
            dosyguidata.residpeak=1;
            set(hEditFlip,'String',num2str(dosyguidata.residpeak))
        end
        set(hEditFlip,'String',num2str(dosyguidata.residpeak))
        guidata(hMainFigure,dosyguidata);
        PlotResidual(dosyguidata.residpeak);
    end
    function ButtonFlipMinus_Callback(source,eventdata)%#ok
        dosyguidata=guidata(hMainFigure);
        dosyguidata.residpeak=dosyguidata.residpeak-1;
        if dosyguidata.residpeak>length(dosydata.freqs)
            dosyguidata.residpeak=length(dosydata.freqs);
            set(hEditFlip,'String',num2str(dosyguidata.residpeak))
        end
        if dosyguidata.residpeak<1
            dosyguidata.residpeak=1;
            set(hEditFlip,'String',num2str(dosyguidata.residpeak))
        end
        set(hEditFlip,'String',num2str(dosyguidata.residpeak))
        guidata(hMainFigure,dosyguidata);
        PlotResidual(dosyguidata.residpeak);
        
    end

%---------End of Plot Residual Callbacks-----------------------------------
    function DrawBoxButton_Callback(source,eventdata) 
        dosyguidata=guidata(hMainFigure);
        zoom off
        pan off
        dosyguidata.boxcounter=0;
        set(hDOSY,'ButtonDownFcn',@DrawBox);
        guidata(hMainFigure,dosyguidata);
    end
    function ClearBoxButton_Callback(source,eventdata) 
        dosyguidata=guidata(hMainFigure);
        dosyguidata.boxcounter=0;
        dosyguidata.boxposcell={};
        dosyguidata.figpos={};
        PlotContour();
        guidata(hMainFigure,dosyguidata);
    end
    function PlotBoxesButton_Callback(source,eventdata) 
        dosyguidata=guidata(hMainFigure);
        ProjectBoxes()
        
    end
    function DrawBox(source,eventdata)  
        dosyguidata=guidata(hMainFigure);
        dosyguidata.boxcounter=(dosyguidata.boxcounter+1);
        boxposrelative=rbbox();
        dosyguidata.figpos{dosyguidata.boxcounter,1} =...
            dsxy2figxy(hDOSY,boxposrelative);
        hRect=rectangle('Parent',hDOSY,...
            'Position', dosyguidata.figpos{dosyguidata.boxcounter,1},...
            'EdgeColor', 'r',...
            'LineWidth', 1);
        guidata(hMainFigure,dosyguidata);
    end
    function ProjectBoxes(source,eventdata) 
        dosyguidata=guidata(hMainFigure);
        for k=1:dosyguidata.boxcounter
            boxpos=dosyguidata.figpos{k,1};
            xpoint1=find(dosydata.Ppmscale>boxpos(1), 1);
            xpoint2=find(dosydata.Ppmscale>(boxpos(1)+boxpos(3)), 1);
            ypoint1=find(dosydata.Dscale>boxpos(2),1);
            ypoint2=find(dosydata.Dscale>(boxpos(2)+boxpos(4)), 1);
            boxes{k,1}=dosydata.DOSY(xpoint1:xpoint2,ypoint1:ypoint2);
            figure
            plot(dosydata.Ppmscale(xpoint1:xpoint2),sum(boxes{k,1}'));
            set(gca,'Xdir','reverse')
            axis(gca,'tight')
            set(gca,'ylim',([(min(min(sum(boxes{k,1}')))-0.1*max(max(sum(boxes{k,1}')))) max(max(sum(boxes{k,1}')))*1.1]));
        end
        guidata(hMainFigure,dosyguidata);
    end
    function varargout = dsxy2figxy(varargin)

    hAx = varargin{1};
	varargin = varargin(2:end); % Remove arg 1 (axes handle)
if length(varargin) == 1       
	pos = varargin{1}; % Extract 4 element position vector
end

% Get limits
axpos = get(hAx,'Position');    % Get axes position
axlim = axis(hAx);               % Get the axis limits [xlim ylim]
axwidth = diff(axlim(1:2));
axheight = diff(axlim(3:4));
 
% Transform from data space coordinates to normalized figure coordinates 
    pos(3) = (pos(3) / axpos(3)) * axwidth;
	pos(4) = (pos(4) / axpos(4)) * axheight;
    pos(1) = (((((axpos(1)+axpos(3))-pos(1))/axpos(3))*axwidth)+ axlim(1))-pos(3);
	pos(2) = (((((axpos(2)+axpos(4))-pos(2))/axpos(4))*axheight)+ axlim(3))-pos(4);
	varargout{1} = pos;

end

    function PlotContour()
        dosyguidata=guidata(hMainFigure);
        xxx=xlim();
        yyy=ylim();
        [xax,yax]=meshgrid(dosydata.Ppmscale,dosydata.Dscale);
        [C,hContour1]=contour(hDOSY,xax,yax,(dosydata.DOSY.*(dosydata.DOSY>0))',dosyguidata.clvl,'b');
        hold(hDOSY,'on')
        [C,hContour2]=contour(hDOSY,xax,yax,(dosydata.DOSY.*(dosydata.DOSY<0))',dosyguidata.clvl,'color',[1 0.5 0]);
        hold(hDOSY,'off')
        set(hContour1,'LineWidth',1)
        set(hContour2,'LineWidth',1)
        set(hDOSY,'LineWidth',1)
        set(hDOSY,'Ydir','reverse');
        ylabel(hDOSY,'\bf Diffusion coefficient m^{2} s^{-1} \times 10^{-10}');
        set(hDOSY,'Xdir','reverse');
        xlabel(hDOSY,'\bf Chemical shift (PPM)');
        ylim(yyy);
        xlim(xxx);
        linkaxes([hDOSY hSpec],'x')
        dosyguidata.hContour1=hContour1;
        dosyguidata.hContour2=hContour2;
        
        guidata(hMainFigure,dosyguidata);
  
    end
    function PlotSpectrum()
        hSpecPlot=plot(hSpec,dosydata.Ppmscale, dosydata.Spectrum,'-b');
        axis(hSpec,'tight')
        axis(hSpec,'off')
        set(hSpec,'Xdir','reverse');
        set(hSpecPlot,'LineWidth',1)
        set(hSpecPlot,'Color','blue')
        
    end
    function PlotProjection()
        hPlotProj=plot(hProj,sum(dosydata.DOSY),dosydata.Dscale,'Visible','Off');
        set(hPlotProj,'LineWidth',2)
        set(hProj,'Ydir','reverse');
        axis(hProj,'tight')
        axis(hProj,'off')
    end
    function PlotResidual(npeak)
        linkaxes([hDOSY hSpec],'off')
        axes(hDOSY)
        get(hMainFigure,'CurrentAxes');
        norm_max=max([max(dosydata.ORIGINAL(npeak,:)) max(dosydata.Gzlvl,dosydata.FITTED(npeak,:))]);
        disp('hi')
        plot(dosydata.Gzlvl,(dosydata.ORIGINAL(npeak,:)./norm_max),'+k','LineWidth',2)
        hold on
        plot(dosydata.Gzlvl,(dosydata.FITTED(npeak,:)./norm_max),'--r','LineWidth',2)
        plot(dosydata.Gzlvl,(dosydata.RESIDUAL(npeak,:)./norm_max),'--b','LineWidth',2)
        set(gca,'LineWidth',2);
        axis('tight')
        ylim('auto')
        ylabel('\fontname{ariel} \bf Amplitude')
        xlabel('\fontname{ariel} \bf Gradient amplitude (T m^{-1})')
        
        str=['\fontname{ariel} \bf Frequency: ' num2str(dosydata.freqs(npeak),3) ' ppm'];
        text(0.5*max(dosydata.Gzlvl),0.95,str);
        nexp=length(find(dosydata.FITSTATS(npeak,:)))/4;
        str=['\fontname{ariel} \bf Number of exponentials: ' num2str(nexp,1)];
        text(0.5*max(dosydata.Gzlvl),0.85,str);
        
        str='\color{black} \bf +  Original';
        text(0.5*max(dosydata.Gzlvl),0.7,str);
        str='\color{red} \bf - -  Fitted';
        text(0.5*max(dosydata.Gzlvl),0.6,str);
        str='\color{blue} \bf - -  Residual';
        text(0.5*max(dosydata.Gzlvl),0.5,str);
        
        hold off
    end




end
%---------------------END of dosyplot_gui--------------------------------------



